package j2d.filters.fftConvolution;

//package j2d.gui;
//package j2d.filters.bufferedImageConvolution;

import gui.ClosableJFrame;
import j2d.ImageUtils;
import j2d.border.BorderUtils;
import j2d.filters.bufferedImageConvolution.ImageMatrixBean;
import j2d.filters.bufferedImageConvolution.ImageMatrixBeanProcessorInterface;
import j2d.gui.ImagePanel;

import javax.media.jai.PlanarImage;
import java.awt.*;
import java.awt.image.BufferedImage;

import utils.PrintUtils;

/**
 * Created by Robert Distinti.
 * User: default
 * Date: Sep 23, 2005
 * Time: 9:06:48 PM
 * <p/>
 * Revision History:
 * 2005 Oct 23 - Modified by Anne MacIlvain to do homework #6.
 */
public class FFTR2ImageConvolverAM implements ImageMatrixBeanProcessorInterface {
    private ImageMatrixBean imb;


    public Image process(Image image) {
        int fftwidth;
        int fftheight;
        BufferedImage bi;
        bi = ImageUtils.getBufferedImage2(image);

        PlanarImage borderImage = BorderUtils.getBorderImage(ImageUtils.getPlanarImage(bi),
                imb.getMatrixSize(), imb.getMatrixSize(), imb.getMatrixSize(), imb.getMatrixSize(),
                imb.getBorderType());

        bi = borderImage.getAsBufferedImage();

        // 2005 Oct 23 - Anne MacIlvain - clip image to even power of 2 size
        fftwidth = clipToPowerOfTwo(bi.getWidth());
        fftheight = clipToPowerOfTwo(bi.getHeight());
        BufferedImage fftbi = bi.getSubimage(0, 0, fftwidth, fftheight);

        // 2005 Oct 23 - Anne MacIlvain
        byte[][] kernelData = imb.getData();
        PrintUtils.displayData(kernelData);

        BufferedImage kernelImage = replicateKernel(fftbi, kernelData);

        Image fftImage = FFTImageAM.fft(fftbi, kernelImage);
        displayImage(fftImage, "This is the PSD of the filtered output");

        Image filteredImage = FFTImageAM.filter(fftbi, kernelImage);

        // Question: How do I open a new window in the desktop?
        // How do I get access to main.java's deskTop member variable??
        // I want to display intermediate results, but I can't.
        displayImage(fftImage, "FFT of Image");

        return filteredImage;
    }

    private void displayImage(Image image, String title) {
        //To change body of created methods use File | Settings | File Templates.
        //j2d.gui.Main.getDesktop();
        System.out.println("I am displaying a new frame here!");
        ClosableJFrame cf = new ClosableJFrame(title);
        Container c = cf.getContentPane();
        ImagePanel ip = new ImagePanel();
        ip.setImage(image);
        c.add(ip);
        cf.setSize(255, 255);
        cf.setVisible(true);

    }

    private BufferedImage replicateKernel(BufferedImage image, byte[][] kernelData) {
        int rgb;
        int kernel;
        int width = image.getWidth();
        int height = image.getHeight();
        int row = kernelData.length;
        int col = kernelData[0].length;
        for (int ww = 0; ww < width; ww++) {
            for (int hh = 0; hh < height; hh++) {
                kernel = kernelData[ww % row][hh % col];
                if (kernel > 0) {
                    rgb = 0xFFFFFF;
                } else {
                    rgb = 0x0;
                }
                image.setRGB(ww, hh, rgb);
            }
        }
        return image;  //To change body of created methods use File | Settings | File Templates.
    }

    private int clipToPowerOfTwo(int data) {
        double exponentTwo = Math.log((double) data) / Math.log(2);
        return (int) Math.pow(2, exponentTwo);  //To change body of created methods use File | Settings | File Templates.
    }

    public FFTR2ImageConvolverAM(ImageMatrixBean o) {
        this.imb = o;
    }
    public String toString(){
        return "FFTR2 convolution";
    }

    public void setImageMatrixBean(ImageMatrixBean imb) {
        this.imb = imb;
    }

    public ImageMatrixBean getImageMatrixBean() {
        return imb;
    }
}

